##########################################
# This file will contain methods to make:
# --ROC diagrams
# --attributes diagrams
# --performance diagrams
# --AUC bar plots
# --BSS bar plots
##########################################

#==========================
# Imports
#==========================

import numpy as np 
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.ticker as ticker


#NOTE: This plan could potentially work
# if we pass in the figure and the subplot
# indices to each function. Then the main
# method could make the plot

#attributes
def makeAttributes(relxs, relys, col_arr, lab_arr, climo, nfcst, ax1):
    x = [0.02, 0.05, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0]

    #make the 1 to 1 line
    #ax.plot(x, x, 'gray', linestyle='dashed', linewidth=4, label='Perfect Reliability')
    ax1.plot(x, x, 'black', linestyle='--', linewidth=5)

    #make the rel curves
    for i in range(len(relxs)):
        #print relxs[i], relys[i]
        #np.isfinite ignores the nan values
        #ignore nan values
        mask1 = np.isfinite(relxs[i]) 
        mask2 = np.isfinite(relys[i])

        if (type(mask1) is np.ndarray):
            ax1.plot(relxs[i][mask1], relys[i][mask2], color=col_arr[i], label='%s' %(lab_arr[i]),linestyle='-',linewidth=5)
        else:
            ax1.plot(relxs[i], relys[i], color=col_arr[i], label='%s' %(lab_arr[i]),linestyle='-',linewidth=5)

    #Add climo lines
    vert_climo_x = [climo, climo]
    vert_climo_y = [0.0, 1.0]

    horiz_climo_x = np.linspace(0.0, 1.0, 11)
    horiz_climo_y = np.ones(len(horiz_climo_x))*climo

    #ax1.plot(vert_climo_x, vert_climo_y, label='Climatological\nFrequency', color='black', linestyle='--', linewidth=4)
    ax1.plot(vert_climo_x, vert_climo_y, color='black', linestyle=':', linewidth=5)
    ax1.plot(horiz_climo_x, horiz_climo_y, linestyle=':',linewidth=5, color='black')

    #add no skill line -- bisects climo and perfect lines. 
    no_skill_x = np.linspace(0.0, 1.0, 11)
    no_skill_y = 0.5*(no_skill_x - climo) + climo
    #ax1.plot(no_skill_x, no_skill_y, label='No Skill', linewidth=4, linestyle='-', color='black' )
    ax1.plot(no_skill_x, no_skill_y, linewidth=5, linestyle='-', color='black' )

    ax1.set_xlim([0.0, 1.0])
    ax1.set_ylim([0.0, 1.0])

    #plt
    #plt.xticks(np.linspace(0.0, 1.0, 6),fontsize=20)
    #plt.yticks(np.linspace(0.0, 1.0, 6),fontsize=20)
    ax1.set_xticks(np.linspace(0.0, 1.0, 6))
    ax1.set_yticks(np.linspace(0.0, 1.0, 6))

    ax1.tick_params(axis="both", labelsize=20)

    #plt
    #plt.ylabel("Obs.\nRel. Freq.", fontsize=20)
    #plt.xlabel("Forecast Probability",fontsize=20)
    ax1.set_ylabel("Obs. Rel. Freq.", fontsize=20)
    ax1.set_xlabel("Forecast Probability",fontsize=20)

    #plt.title(titlename,fontsize=22,fontweight='bold')

    #plt.legend(fontsize=20, loc='upper right')

    #add the number of forecast plots 
    ax2 = ax1.twinx()
    #nlabs = ['# Raw Forecasts', '# Smooth Forecasts', '# RF Forecasts']

    #make y-axis logarithmic 
    ax2.set_yscale('log')
    ax2.set_ylim([10**2, 10**7])
    for i in range(len(relxs)):
        ax2.plot(relxs[i], nfcst[i], color=col_arr[i], linewidth=2,linestyle='--', marker='o')

    #plt 
    ax2.set_ylabel("# Forecasts", fontsize=20)
    ax2.tick_params(axis="y", labelsize=20) 

    return


#ROC
def makeROC(pofds, pods, aucs, col_arr, lab_arr, ax1):
    for i in range(len(pods)):
        rounded_auc = format(float(aucs[i]), '.3f')
        lab = lab_arr[i] + " (AUC: %s)" %rounded_auc
        ax1.plot(pofds[i], pods[i], color=col_arr[i], label='%s' %(lab),linestyle='-',linewidth=4)

    ax1.set_xlim([0.0, 1.0])
    ax1.set_ylim([0.0, 1.0])

    #make and plot the 1-1 (no skill) line
    x1 = np.arange(0, 1.1, 0.1)
    ax1.plot(x1, x1, '--k', linewidth=4, label='No Skill')
    plt.xticks(np.linspace(0.0, 1.0, 6), fontsize=17)
    plt.yticks(np.linspace(0.0, 1.0, 6), fontsize=17)
    plt.ylabel("POD", fontsize=17)
    plt.xlabel("POFD",fontsize=17)
    #plt.title(titlename,fontsize=22, fontweight='bold')
    
    plt.legend(fontsize=14, loc=4)


    return


#perfDiag
def makePerfDiag(srs, pods, col_arr, lab_arr, ax1):

    #Obtain bias and CSI lines
    #plot CSI lines from 0.1 to 1.
    csi = np.linspace(0.1, 1., 10)

    #plot bias from 0.1 to 1, 1.3, 1.5, 2., 3., 5., 10.
    #bias = np.array((0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0, 1.3, 1.5, 2.0, 3.0, 5.0, 10.0))

    #bias = np.array((0.2, 0.4, 0.6, 0.8, 1.0, 1.5, 2.0, 3.0, 5.0, 10.0))
    bias = np.array((0.25, 0.5, 0.75, 1.0, 1.5,  2.0, 3.0, 5.0, 10.0))


    #get points to plot for csi and bias in terms of POD. 
    sr_vals = np.linspace(0., 1.0, 101) #Will vary the SR values and obtain a pod value

    POD_csi = np.zeros((len(csi), len(sr_vals)))
    POD_bias = np.zeros((len(bias), len(sr_vals)))


    for j in range(len(sr_vals)):
        for i in range(len(csi)):
            POD_csi[i,j] = 1/(1/csi[i] - 1/sr_vals[j] + 1)
        for i in range(len(bias)):
            POD_bias[i,j] = bias[i]*sr_vals[j]

    #Replace values less than 0 with NaNs

    POD_csi = np.where(POD_csi < 0., np.nan, POD_csi)


    #plot the CSI and bias lines

    for i in range(len(csi)):
        plt.plot(sr_vals, POD_csi[i,:], 'k-')
       
#        if (i < len(csi)-1): 
#            plt.fill_between(sr_vals, POD_csi[i+1,:], 'blue', alpha=0.5)

    for i in range(len(bias)):
        plt.plot(sr_vals, POD_bias[i,:],'k--')


    plt.xticks(np.linspace(0., 1., 6), fontsize=20)
    plt.yticks(np.linspace(0.,1.,6), fontsize=20)
    plt.xlim(0., 1.)
    plt.ylim(0.,1.)
    plt.xlabel("Success Ratio", fontsize = 20)
    plt.ylabel("POD", fontsize=20)

    #Plot the raw, smooth, and RF perf diags
    for i in range(len(srs)):
        #[1:-1] because want to exclude 0% and 100% thresholds 
        plt.plot(srs[i], pods[i], color=col_arr[i], linestyle='-', marker='o', label=lab_arr[i], linewidth=4)

    #plt.legend(loc='upper right', fontsize=20)

    #Label the csi lines
    #ax1.text(0.14, 0.28, "0.1", fontsize=12, rotation=-25, fontweight='bold',transform=ax1.transAxes)
    #ax1.text(0.26, 0.44, "0.2", fontsize=12, rotation=-30, fontweight='bold',transform=ax1.transAxes)
    #ax1.text(0.37, 0.59, "0.3", fontsize=12, rotation=-40, fontweight='bold',transform=ax1.transAxes)
    #ax1.text(0.48, 0.72, "0.4", fontsize=12, rotation=-45, fontweight='bold',transform=ax1.transAxes)
    #ax1.text(0.57, 0.8, "0.5", fontsize=12, rotation=-45, fontweight='bold',transform=ax1.transAxes)
    #ax1.text(0.64, 0.72, "0.5", fontsize=12, rotation=-35, fontweight='bold',transform=ax1.transAxes)
    #ax1.text(0.65, 0.87, "0.6", fontsize=12, rotation=-35,fontweight='bold',transform=ax1.transAxes)
    #ax1.text(0.73, 0.95, "0.7", fontsize=12, rotation=-45,fontweight='bold',transform=ax1.transAxes)
    #ax1.text(0.85, 0.94, "0.8", fontsize=12, rotation=-35, fontweight='bold', transform=ax1.transAxes)
    #ax1.text(0.9, 1.0, "0.9", fontsize=12, rotation=-40, fontweight='bold', transform=ax1.transAxes)


    #Label the bias lines: 0.25, 0.5, 0.75, 1.0, 1.5,  2.0, 3.0, 5.0, 10.0

    #ax1.text(1.02, 0.23, "0.25", fontsize=12, fontweight='bold',transform=ax1.transAxes)
    #ax1.text(1.02, 0.48, "0.5", fontsize=12,  fontweight='bold',transform=ax1.transAxes)
    #ax1.text(1.02, 0.73, "0.75", fontsize=12, fontweight='bold',transform=ax1.transAxes)
    #ax1.text(1.02, 1.01, "1", fontsize=12, fontweight='bold',transform=ax1.transAxes)
    #ax1.text(0.6, 1.02, "1.5", fontsize=12, fontweight='bold',transform=ax1.transAxes)
    #ax1.text(0.48, 1.02, "2", fontsize=12, fontweight='bold',transform=ax1.transAxes)
    #ax1.text(0.32, 1.02, "3", fontsize=12, fontweight='bold',transform=ax1.transAxes)
    #ax1.text(0.18, 1.02, "5", fontsize=12, fontweight='bold', transform=ax1.transAxes)
    #ax1.text(0.04, 1.02, "10", fontsize=12, fontweight='bold', transform=ax1.transAxes)
    #ax1.text(0.01, 0.9, "10", fontsize=12, fontweight='bold', rotation=80, transform=ax1.transAxes)

    return


#AUC bar plot

#Can be used to make bar plots for AUC, BSS, BSrel, BSres, BSunc
def makeBarPlots(AUCs_sref, AUCs_href, col_arr, lab_arr, ens_arr, ymin, ymax, ylab, isLog, isSciNot, ax):
#lab_arr is Raw, Smooth, RF
#ens_arr is HREFv2, SREF

    if (isLog == True):
        ax.set_yscale('log')

    ax.set_ylim([ymin, ymax])
    bar_width = 0.32
    opacity = 0.9
    index = np.arange(len(ens_arr))

    AUCs = [AUCs_sref, AUCs_href]

    label_positions = []
    #loop over the SREF, HREFv2
    for v in range(len(ens_arr)):

    #loop over the raw, smooth, RF
        for i in range(len(AUCs_href)):
            height = AUCs[v][i]
            if i == 1:
                label_positions.append(index[v] + bar_width*i)
            if v == 0:

                plt.bar(index[v] + bar_width*i, height, bar_width, alpha=opacity, color=col_arr[i],  label=lab_arr[i])
            else:
                plt.bar(index[v] + bar_width*i, height, bar_width, alpha=opacity, color=col_arr[i])

            #print AUC on top of the bar
            if (isSciNot == True):
                #For paper:
                #plt.text(index[v] + bar_width*i, height, "%.1E" %height, ha='center', va='bottom', fontsize=14)
                plt.text(index[v] + bar_width*i, height, "%.1E" %height, ha='center', va='bottom', fontsize=13) 

            else:
                #For paper:
                #plt.text(index[v] + bar_width*i, height, "%.3f" %height, ha='center', va='bottom', fontsize=14)
                plt.text(index[v] + bar_width*i, height, "%.3f" %height, ha='center', va='bottom', fontsize=17)

    #plt.xlabel("Forecast", fontsize=24)
    #plt.ylabel(ylab, fontsize=24)
    plt.ylabel(ylab, fontsize=24, fontweight='bold')
    plt.yticks(fontsize=20)
    plt.xticks(label_positions, ens_arr,fontsize=20)
    #plt.legend(fontsize=18, loc='upper left') 
    return

#Make the monthly bar plots 
#e.g., col_arr is ["red", "blue"] 
#lab_arr is ["rf", "spc"]
#month_labels is ["Apr.", "May", "Jun." ... ]
def monthly_barplot(metrics_rf, metrics_spc, col_arr, lab_arr, month_labels, ymin, ymax, ylab, isSciNot, ax):
    bar_width = 0.32
    opacity = 0.9
    index = np.arange(len(month_labels))

    ax.set_ylim([ymin, ymax])

    metrics = [metrics_rf, metrics_spc]

    label_positions = []

    #loop over month
    for v in range(len(month_labels)):
        #loop over months
        for i in range(len(lab_arr)):
            height = metrics[i][v]

            if i == 1:
                label_positions.append(index[v] + bar_width*i/2)
            if v == 0:
                plt.bar(index[v] + bar_width*i, height, bar_width, alpha=opacity, color=col_arr[i],  label=lab_arr[i])
            else:
                plt.bar(index[v] + bar_width*i, height, bar_width, alpha=opacity, color=col_arr[i])

            #print AUC on top of the bar
            if (isSciNot == True):
                #For paper:
                #plt.text(index[v] + bar_width*i, height, "%.1E" %height, ha='center', va='bottom', fontsize=14)
                plt.text(index[v] + bar_width*i, height, "%.1E" %height, ha='center', va='bottom', fontsize=13)

            else:
                #For paper:
                #plt.text(index[v] + bar_width*i, height, "%.3f" %height, ha='center', va='bottom', fontsize=14)
                plt.text(index[v] + bar_width*i, height, "%.2f" %height, ha='center', va='bottom', fontsize=13)

    #plt.xlabel("Forecast", fontsize=24)
    #plt.ylabel(ylab, fontsize=24)
    plt.ylabel(ylab, fontsize=24, fontweight='bold')
    plt.yticks(fontsize=20)
    plt.xticks(label_positions, month_labels, fontsize=20)
    plt.legend(fontsize=18, loc='upper left') 


    return


#Make the monthly bar plots 
#e.g., col_arr is ["red", "blue"] 
#lab_arr is ["rf", "spc"]
#month_labels is ["Apr.", "May", "Jun." ... ]
#NOTE metrics is in form (months, n_ens/forecast types, appropriate length of metric) 
#n_samples is a month-dimension array showing sample size (i.e., how many dates are in each month/"bin") 
#showClimo determines whether or not to show sample climatology on twin axis
def monthly_barplot_w_uh(metrics, col_arr, lab_arr, month_labels, n_samples, ymin, ymax, ylab, isSciNot, showClimo, ax):
    bar_width = 0.15
    opacity = 0.9
    index = np.arange(len(month_labels))

    ax.set_ylim([ymin, ymax])

    if (isSciNot == True):
        ax.set_yscale('log')


    #metrics = [metrics_rf, metrics_spc]

    label_positions = []

    #loop over month
    for v in range(len(month_labels)):
        #loop over forecast types
        for i in range(len(lab_arr)):
            #height = metrics[i][v]
            height = metrics[v][i] 

            if i == 1:
                label_positions.append(index[v] + bar_width*2)
            if v == 0:
                plt.bar(index[v] + bar_width*i, height, bar_width, alpha=opacity, color=col_arr[i],  label=lab_arr[i])
            else:
                plt.bar(index[v] + bar_width*i, height, bar_width, alpha=opacity, color=col_arr[i])

            #print AUC on top of the bar
            #if (isSciNot == True):
                #For paper:
                #plt.text(index[v] + bar_width*i, height, "%.1E" %height, ha='center', va='bottom', fontsize=14)
            #    plt.text(index[v] + bar_width*i, height, "%.1E" %height, ha='center', va='bottom', fontsize=13)

            #else:
                #For paper:
                #plt.text(index[v] + bar_width*i, height, "%.3f" %height, ha='center', va='bottom', fontsize=14)
            #    plt.text(index[v] + bar_width*i, height, "%.2f" %height, ha='center', va='bottom', fontsize=13)


    #plt.xlabel("Forecast", fontsize=24)
    #plt.ylabel(ylab, fontsize=24)
    plt.ylabel(ylab, fontsize=20, fontweight='bold')
    plt.yticks(fontsize=18)
    plt.xticks(label_positions, month_labels, fontsize=18)
    #plt.legend(fontsize=18, loc='upper right', ncol=3)


    #Twin axis ; Plot sample size over the top with a black dashed line
    if (showClimo == True):
        ax2 = ax.twinx()

        ax2.set_ylim([0, 0.05])
        ax2.plot(index + bar_width, n_samples, color='black', linewidth=2,linestyle='--', marker='o', label="Sample Climatology") 

        plt.ylabel("Sample Climatology", fontsize=20)
        plt.yticks(fontsize=18)
        plt.legend(fontsize=18, loc='upper right') 


    return




#For making the seasonal/regional plots: 

#e.g., col_arr is ["red", "blue"] 
#lab_arr is ["rf", "spc"]
#month_labels is ["Apr.", "May", "Jun." ... ]
#NOTE metrics is in form (months, n_ens/forecast types, appropriate length of metric) 
#n_samples is a month-dimension array showing sample size (i.e., how many dates are in each month/"bin") 
#NOTE: Now we will split the diagram to show full range of data
#ax_break is the second axis corresponding to the break points
#set_samp_size is if we should plot the sample size on a twin axis. 
def monthly_barplot_w_uh_and_split(metrics, col_arr, lab_arr, month_labels, n_samples, ymin, ymax, ylab, isSciNot, set_samp_size, breakPoints, ax, ax_break):
    bar_width = 0.09
    opacity = 0.9
    index = np.arange(len(month_labels))

    #ax.set_ylim([ymin, ymax])

    #metrics = [metrics_rf, metrics_spc]

    label_positions = []

    print (metrics.shape)

     #Add two subplots

    ax.set_ylim([ymin,ymax])
    ax_break.set_ylim(breakPoints)

    #loop over month
    for v in range(len(month_labels)):
        #loop over forecast types
        for i in range(len(lab_arr)):
            #height = metrics[i][v]
            height = metrics[v][i]

            if i == 1:
                label_positions.append(index[v] + bar_width*4)
            if v == 0:
                ax.bar(index[v] + bar_width*i, height, bar_width, alpha=opacity, color=col_arr[i],  label=lab_arr[i])
                ax_break.bar(index[v] + bar_width*i, height, bar_width, alpha=opacity, color=col_arr[i],  label=lab_arr[i])
            else:
                ax.bar(index[v] + bar_width*i, height, bar_width, alpha=opacity, color=col_arr[i])
                ax_break.bar(index[v] + bar_width*i, height, bar_width, alpha=opacity, color=col_arr[i])

            #print AUC on top of the bar
            #if (isSciNot == True):
                #For paper:
                #plt.text(index[v] + bar_width*i, height, "%.1E" %height, ha='center', va='bottom', fontsize=14)
            #    plt.text(index[v] + bar_width*i, height, "%.1E" %height, ha='center', va='bottom', fontsize=13)

            #else:
                #For paper:
                #plt.text(index[v] + bar_width*i, height, "%.3f" %height, ha='center', va='bottom', fontsize=14)
            #    plt.text(index[v] + bar_width*i, height, "%.2f" %height, ha='center', va='bottom', fontsize=13)


    #plt.xlabel("Forecast", fontsize=24)
    #plt.ylabel(ylab, fontsize=24)
    #plt.ylabel(ylab, fontsize=24, fontweight='bold')
    #plt.yticks(fontsize=20)
    #plt.xticks(label_positions, month_labels, fontsize=20)
    #plt.legend(fontsize=18, loc='upper left', ncol=3)


    ax_break.spines['top'].set_visible(False)
    ax.spines['bottom'].set_visible(False)

    ax.axes.get_xaxis().set_ticks([])
    ax_break.axes.get_xaxis().set_ticks([])

    d = 0.01
    kwargs = dict(transform=ax.transAxes, color='k', clip_on=False)
    ax.plot((-d, +d), (-d, +d), **kwargs)
    ax.plot((1 - d, 1 + d), (-d, +d), **kwargs)  # top-right diagonal

    kwargs.update(transform=ax_break.transAxes)  # switch to the bottom axes
    ax_break.plot((-d, +d), (1 - d, 1 + d), **kwargs)  # bottom-left diagonal
    ax_break.plot((1 - d, 1 + d), (1 - d, 1 + d), **kwargs)  # bottom-right diagonal

    ax.set_ylabel(ylab, fontsize=20, fontweight='bold')

    plt.xticks(label_positions, month_labels, fontsize=18)

    ax_break.tick_params(axis='x', which='major', labelsize=18, top=False)

    ax.tick_params(axis='y', which='major', labelsize=18)
    ax_break.tick_params(axis='y', which='major', labelsize=18)

    if (set_samp_size == True):

        #Twin axis ; Plot sample size over the top with a black dashed line
        ax2 = ax.twinx()

        ax2.set_ylim([0, 650])
        ax2.plot(index + bar_width*4, n_samples, color='black', linewidth=2,linestyle='--', marker='o', label="Sample Size")

        plt.ylabel("Sample Size\n(# Forecast Days)", fontsize=20)
        plt.yticks(fontsize=18)
        #plt.legend(fontsize=18, loc='upper right') 
        return ax2

    else:

        return ax_break






#Below is for making the bss barplots (e.g., w RF, RF_trunc, SPC orig, SPC full, and UH Best)

#makes monthly barplot w uh with split axis 
#ax2 is the lower axis. 
#breakPoints is a tuple indicating the ylimits of the second axis (e.g., (0.78, 1.0) )
def monthly_barplot_w_uh_w_split(metrics, col_arr, lab_arr, month_labels, n_samples, ymin, ymax, ylab, isSciNot, showClimo, breakPoints, ax, ax2):

    bar_width = 0.15
    opacity = 0.9
    index = np.arange(len(month_labels))

    ax.set_ylim([ymin,ymax])
    ax2.set_ylim(breakPoints)

    if (isSciNot == True):
        ax.set_yscale('log')
        ax2.set_yscale('log') 

    #metrics = [metrics_rf, metrics_spc]

    label_positions = []

    #loop over month
    for v in range(len(month_labels)):
        #loop over forecast types
        for i in range(len(lab_arr)):
            #height = metrics[i][v]
            height = metrics[v][i]

            if i == 1:
                label_positions.append(index[v] + bar_width*2)
            if v == 0:
                ax.bar(index[v] + bar_width*i, height, bar_width, alpha=opacity, color=col_arr[i],  label=lab_arr[i])
                ax2.bar(index[v] + bar_width*i, height, bar_width, alpha=opacity, color=col_arr[i],  label=lab_arr[i])
            else:
                ax.bar(index[v] + bar_width*i, height, bar_width, alpha=opacity, color=col_arr[i])
                ax2.bar(index[v] + bar_width*i, height, bar_width, alpha=opacity, color=col_arr[i])


        ax2.spines['top'].set_visible(False)
        ax.spines['bottom'].set_visible(False)
        #ax.xaxis.tick_top()
        #ax.tick_params(axis='x', top=False, bottom=False, which='both')
        #ax2.tick_params(axis='x', which='both') 

        ax.axes.get_xaxis().set_ticks([])
        ax2.axes.get_xaxis().set_ticks([])
        #ax.tick_params(bottom=False,top=False, labeltop='off', labelbottom='off') 
        #ax2.tick_params(axis='x', labeltop='off', labelbottom='off', bottom=False, top=False, which='minor') 
        #ax2.xaxis.tick_bottom()
        #ax.spines['bottom'].set_visible(False)
        #ax2.spines['top'].set_visible(False)
        #ax2.xaxis.tick_bottom()
        #ax2.tick_params(labelbottom='on') 
        #ax.tick_params(labeltop='off') 
        #ax.xaxis.tick_top()

        d = 0.01
        kwargs = dict(transform=ax.transAxes, color='k', clip_on=False)
        ax.plot((-d, +d), (-d, +d), **kwargs)
        ax.plot((1 - d, 1 + d), (-d, +d), **kwargs)  # top-right diagonal

        kwargs.update(transform=ax2.transAxes)  # switch to the bottom axes
        ax2.plot((-d, +d), (1 - d, 1 + d), **kwargs)  # bottom-left diagonal
        ax2.plot((1 - d, 1 + d), (1 - d, 1 + d), **kwargs)  # bottom-right diagonal

        ax.set_ylabel(ylab, fontsize=24, fontweight='bold')

        plt.xticks(label_positions, month_labels)


        #Set yticks in scientific notation
        f = ticker.ScalarFormatter(useOffset=False, useMathText=True)
        g = lambda x,pos : "${}$".format(f._formatSciNotation('%1.1e' % x))
        ax.yaxis.set_major_formatter(ticker.FuncFormatter(g)) 
        ax2.yaxis.set_major_formatter(ticker.FuncFormatter(g))

        ax2.tick_params(axis='x', which='major', labelsize=20, top=False)

        ax.tick_params(axis='y', which='major', labelsize=20)
        ax2.tick_params(axis='y', which='major', labelsize=20)


        #ax.legend(fontsize=18,loc='upper right', ncol=3) 

    #Twin axis ; Plot sample size over the top with a black dashed line
    if (showClimo == True):
        ax12 = ax.twinx()

        ax12.set_ylim([0, 0.05])
        ax12.plot(index + bar_width, n_samples, color='black', linewidth=2,linestyle='--', marker='o', label="Sample Climatology")

        plt.ylabel("Sample Climatology", fontsize=20)
        plt.yticks(fontsize=20)
        plt.legend(fontsize=18, loc='upper right')

    return 


#Separates an exponential value into two lines. 
#E.g., converts 2.953E-3 to 2.953
#                            E-3
#in_val is a float
def format_expon(in_val):
    
    out_val = format(in_val, ".2E") 

    out_val = out_val.split("E")
    new_out_val = "%s\nE%s" %(out_val[0], out_val[1]) 

    return new_out_val

#Makes a single-dimension barplot with a break or split in the axis
#breakPoints is a tuple indicating the ylimits of the second axis (e.g., (0.78, 1.0) )
#NOTE: For this one, do NOT feed in a pre-made ax; will create one inside this method
def single_dim_barplot_w_split(metrics, col_arr, lab_arr, ymin, ymax, ylab, isSciNot, breakPoints, ax, ax2):

    bar_width = 0.9
    opacity = 0.9

    #index = np.arange(len(metrics)) 

    index = np.arange(len(lab_arr) )

    label_positions = []

    #ax = fig.add_subplot(2,2,1)

    #ax2 = plt.subplot(122, sharex = ax)

    #Add two subplots

    ax.set_ylim([ymin,ymax])
    ax2.set_ylim(breakPoints)

    #TODO: Plot the same data on both axes

    for v in range(len(lab_arr)):

        height = metrics[v]
        label_positions.append(index[v] + bar_width*v)
        ax.bar(index[v] + bar_width*v, height, bar_width, alpha=opacity, color=col_arr[v])
        ax2.bar(index[v] + bar_width*v, height, bar_width, alpha=opacity, color=col_arr[v])

        if (isSciNot == False):
            if (height >= 0):
                
                if (height > breakPoints[1]):
                    ax.text(index[v] + bar_width*v, height, "%.3f" %height, ha='center', va='bottom', fontsize=14)
                if (height <= breakPoints[1]): 
                    ax2.text(index[v] + bar_width*v, height, "%.3f" %height, ha='center', va='bottom', fontsize=14)
            else:
                if (height > breakPoints[1]):
                    ax.text(index[v] + bar_width*v, height, "%.3f" %height, ha='center', va='top', fontsize=14)

                if (height <= breakPoints[1]):
                    ax2.text(index[v] + bar_width*v, height, "%.3f" %height, ha='center', va='top', fontsize=14)
        else:
            str_height = format_expon(height) 

            if (height >= 0):

                if (height > breakPoints[1]):
                    #ax.text(index[v] + bar_width*v, height, "%.2E" %height, ha='center', va='bottom', fontsize=14)
                    ax.text(index[v] + bar_width*v, height, str_height, ha='center', va='bottom', fontsize=14)
                if (height <= breakPoints[1]):
                    #ax2.text(index[v] + bar_width*v, height, "%.2E" %height, ha='center', va='bottom', fontsize=14)
                    ax2.text(index[v] + bar_width*v, height, str_height, ha='center', va='bottom', fontsize=14)

            else:
                if (height > breakPoints[1]):
                    #ax.text(index[v] + bar_width*v, height, "%.2E" %height, ha='center', va='top', fontsize=14)
                    ax.text(index[v] + bar_width*v, height, str_height, ha='center', va='top', fontsize=14)
                if (height <= breakPoints[1]):
                    #ax2.text(index[v] + bar_width*v, height, "%.2E" %height, ha='center', va='top', fontsize=14)
                    ax2.text(index[v] + bar_width*v, height, str_height, ha='center', va='top', fontsize=14)

        ax2.spines['top'].set_visible(False)
        ax.spines['bottom'].set_visible(False)
        #ax.xaxis.tick_top()
        #ax.tick_params(axis='x', top=False, bottom=False, which='both')
        #ax2.tick_params(axis='x', which='both') 


        ax.axes.get_xaxis().set_ticks([])
        ax2.axes.get_xaxis().set_ticks([])
        

        #ax.tick_params(bottom=False,top=False, labeltop='off', labelbottom='off') 
        #ax2.tick_params(axis='x', labeltop='off', labelbottom='off', bottom=False, top=False, which='minor') 
        #ax2.xaxis.tick_bottom()

        #ax.spines['bottom'].set_visible(False)
        #ax2.spines['top'].set_visible(False)
        #ax2.xaxis.tick_bottom()
        #ax2.tick_params(labelbottom='on') 
        #ax.tick_params(labeltop='off') 
        #ax.xaxis.tick_top()

        d = 0.015
        kwargs = dict(transform=ax.transAxes, color='k', clip_on=False)
        ax.plot((-d, +d), (-d, +d), **kwargs)
        ax.plot((1 - d, 1 + d), (-d, +d), **kwargs)  # top-right diagonal

        kwargs.update(transform=ax2.transAxes)  # switch to the bottom axes
        ax2.plot((-d, +d), (1 - d, 1 + d), **kwargs)  # bottom-left diagonal
        ax2.plot((1 - d, 1 + d), (1 - d, 1 + d), **kwargs)  # bottom-right diagonal

        ax.set_ylabel(ylab, fontsize=24, fontweight='bold')

        plt.xticks(label_positions, lab_arr)

        ax2.tick_params(axis='x', which='major', labelsize=24, top=False)

        ax.tick_params(axis='y', which='major', labelsize=24)
        ax2.tick_params(axis='y', which='major', labelsize=24)


    return


#makes a single-dimension barplot (e.g., BSS for 12 months) 
#breakAxis is boolean. if True, breaks the axis
#breakPoints is a tuple indicating the ylimits of the second axis (e.g., (0.78, 1.0) ) 
def single_dim_barplot(metrics, col_arr, lab_arr, ymin, ymax, ylab, isSciNot, ax):

    bar_width = 0.9
    opacity = 0.9

    #index = np.arange(len(metrics)) 

    index = np.arange(len(lab_arr) )

    label_positions = []

    ax.set_ylim([ymin, ymax]) 

    for v in range(len(lab_arr)): 

        height = metrics[v] 
        label_positions.append(index[v] + bar_width*v) 
        ax.bar(index[v] + bar_width*v, height, bar_width, alpha=opacity, color=col_arr[v])
        if (isSciNot == False):
            if (height >= 0): 
                ax.text(index[v] + bar_width*v, height, "%.3f" %height, ha='center', va='bottom', fontsize=14)
            else: 
                ax.text(index[v] + bar_width*v, height, "%.3f" %height, ha='center', va='top', fontsize=14)

        else:
            str_height = format_expon(height) #split into two rows for better display
            if (height >= 0):
                #ax.text(index[v] + bar_width*v, height, "%.2E" %height, ha='center', va='bottom', fontsize=14)
                ax.text(index[v] + bar_width*v, height, str_height, ha='center', va='bottom', fontsize=14)
            else: 
                #ax.text(index[v] + bar_width*v, height, "%.2E" %height, ha='center', va='top', fontsize=14)
                ax.text(index[v] + bar_width*v, height, str_height, ha='center', va='top', fontsize=14)

    ax.set_ylabel(ylab, fontsize=24, fontweight='bold') 
    
    plt.xticks(label_positions, lab_arr) 

    ax.tick_params(axis='x', which='major', labelsize=24)
    ax.tick_params(axis='y', which='major', labelsize=24) 

    return


#Makes a single plot but shows different metrics on the same axes -- including
#AUC, BSS, BS_rel, and BS_res
def multi_dim_barplot(aucs, bsses, bs_rels, bs_reses, col_arr, lab_arr, ax):
    bar_width = 0.32
    opacity = 0.9
    
    metrics = [aucs, bsses, bs_rels, bs_reses] 
    metric_names = ["AUC", "BSS", "BS rel.", "BS res." ] 

    index = np.arange(len(metrics))

    label_positions = []

    ax.set_ylim([0.,1.]) 

    ax2 = ax.twinx()
    #make y-axis logarithmic 
    ax2.set_yscale('log')
    ax2.set_ylim([10**-5, 10**-1])

    for v in range(len(metrics)):
        for i in range(len(aucs)):
            height = metrics[v][i]
           
            if i == 1:
                label_positions.append(index[v] + bar_width*i/2) 

            #print height value on top of the var
            if (v > 1):
                if (v == 2):
                    ax2.bar(index[v] + bar_width*i, height, bar_width, alpha=opacity, color=col_arr[i], label=lab_arr[i])
                else: 
                    ax2.bar(index[v] + bar_width*i, height, bar_width, alpha=opacity, color=col_arr[i]) 

                ax2.text(index[v] + bar_width*i, height, "%.1E" %height, ha='center', va='bottom', fontsize=13)

            else:
                if (v == 0):
                    ax.bar(index[v] + bar_width*i, height, bar_width, alpha=opacity, color=col_arr[i], label=lab_arr[i])
                else:
                    ax.bar(index[v] + bar_width*i, height, bar_width, alpha=opacity, color=col_arr[i]) 

                ax.text(index[v] + bar_width*i, height, "%.3f" %height, ha='center', va='bottom', fontsize=13) 

    ax.set_ylabel("Metric Score/Value", fontsize=24, fontweight='bold')
    ax2.set_ylabel("Metric Score/Value", fontsize=24, fontweight='bold') 
    #plt.yticks(fontsize=20)
    plt.xticks(label_positions, metric_names)
    
    ax.tick_params(axis='x', which='major', labelsize=24)
    ax2.tick_params(axis='y', which='major', labelsize=24)
    ax.tick_params(axis='y', which='major', labelsize=24) 

    plt.legend(fontsize=14, loc='upper center')

    return







#BSS bar plot 
def makeBSSbar(BSSs_sref, BSSs_href, col_arr, lab_arr, ens_arr, ax):
    return

#BSS reliability bar plot
def makeBSrel():
    return


#BSS resolution bar plot
def makeBSres():
    return




#BSS uncertainty bar plot
def makeBSunc():
    return









