import numpy as np 
from sklearn.metrics import brier_score_loss, roc_auc_score, auc
#==============================================
# Calculate Brier Score and Brier Skill Score 
#==============================================

#Maybe use this a little later in place of f_star. 
probs_auc = np.zeros(23) # will be a set including: 0,1,2,5,10-100 by 5s. 
#probs_rel = np.zeros(12) # will be a set including: 0, 5, 15-95 by 10s, 100
probs_rel = np.zeros(7) 
#0,1,2,5,10-100 by 5s
#probs_auc[0] = 0.00 ; probs_auc[1] = 0.01 ; probs_auc[2] = 0.02 ; probs_auc[3] = 0.05
#for i in range(4,len(probs_auc)):
#    probs_auc[i] = float((i-2)*0.05)

#0, 5, 15-95 by 10s, 100
#probs_rel[0] = 0.00 ; probs_rel[1] = 0.05 ; probs_rel[-1] = 1.00
#for i in range(2, len(probs_rel)-1):
#    probs_rel[i] = float(0.05 + (i-1)*0.10)

#probs_rel[0] = 0.00 ; probs_rel[1] = 0.05 ; probs_rel[2] = 0.15 ; probs_rel[3] = 0.30 ; probs_rel[4] = 0.45 ; probs_rel[5] = 0.60 ; probs_rel[6] = 1.

#probs_auc = np.array([0.00, 0.05, 0.15, 0.30, 0.45, 0.60, 0.75, 1.0])
#probs_rel = np.array([0.00, 0.05, 0.15, 0.30, 0.45, 0.60, 0.75, 1.0])

#probs_auc = np.array([0.00, 0.02, 0.05, 0.10, 0.15, 0.20, 0.25, 0.30, 0.35, 0.40, 0.45, 0.50, 0.55, 0.60, 0.65, 0.70, 0.75, 0.80, 0.85, 0.90, 0.95, 1.0]) 
#probs_rel = np.array([0.00, 0.02, 0.05, 0.10, 0.15, 0.20, 0.25, 0.30, 0.35, 0.40, 0.45, 0.50, 0.55, 0.60, 0.65, 0.70, 0.75, 0.80, 0.85, 0.90, 0.95, 1.0])

probs_auc = np.array([0.00, 0.01, 0.02, 0.05, 0.15, 0.25, 0.35, 0.45, 0.55, 0.65, 0.75, 0.85, 0.95, 1.0]) 
probs_rel = np.array([0.00, 0.01, 0.02, 0.05, 0.15, 0.25, 0.35, 0.45, 0.55, 0.65, 0.75, 0.85, 0.95, 1.0])


#separates the forecast and obs into k bins based on forecast probability
#def get_bins(y_test, y_train, predictions):
def get_bins(y_test,predictions):    
    binned_preds = [[] for i in range(len(probs_rel))] #binned predictions
    binned_ytest = [[] for i in range(len(probs_rel))] #binned y_test
    #binned_ytrain = [[] for i in range(len(probs))] #binned y_train
    
    #put the data into bins.
    for i in range(len(predictions)):
        for p in range(len(probs_rel)-1):
            if (predictions[i] >= probs_rel[p] and predictions[i] < probs_rel[p+1]):
                binned_preds[p].append(predictions[i])
                binned_ytest[p].append(y_test[i])
                #binned_ytrain[p].append(y_train[i])
                break
            elif (predictions[i] == probs_rel[-1]): #include a 1.00 prob in the last bin
                binned_preds[-1].append(predictions[i])  
                binned_ytest[-1].append(y_test[i])
                #binned_ytrain[-1].append(y_train[i])
                break

    #return binned_preds, binned_ytest, binned_ytrain
    return binned_preds, binned_ytest

#Decomposes brier score (of testing set) into reliability, resolution, and uncertainty terms
#def BS_decomposition(y_test, y_train, predictions):
def BS_decomposition(y_test, predictions):
    N = len(predictions) # number of forecasts
    k_bins = len(probs_rel) 
       
    #get binned data
    #bin_preds,bin_ytest,bin_ytrain = get_bins(y_test,y_train,predictions)
    bin_preds,bin_ytest = get_bins(y_test,predictions)    

    #Compute o_bar, the observed climatological base rate -- using testing data (correct?)
    o_bar = np.mean(y_test)

    #Compute the uncertainty term
    uncertainty = o_bar*(1-o_bar)

    #Compute Reliability and resolution terms
    reliability = 0.0
    resolution = 0.0
    for i in range(k_bins):
        n = len(bin_preds[i])
        if (n > 0):
            p = np.mean(bin_preds[i])
            o = np.mean(bin_ytest[i]) #Ok_bar
            reliability += (n*(p-o)**2)
            resolution += (n*(o-o_bar)**2)
        
    reliability = reliability/float(N)
    resolution = resolution/float(N)    

    return reliability,resolution,uncertainty

#computes ETS (at different thresholds of fcst probs.) given
#a set of forecast probs and a set of observations
def ETS(forecast_probs, observation):
    #f_star = np.arange(0, 1.01, 0.01) # levels of probability to analyze
    f_star = probs_auc
    ets = np.zeros((len(f_star)))

    for i in range(len(f_star)):
        prob_threshold = f_star[i]
        bin_forecast = binarize_prob_forecast(forecast_probs, prob_threshold)
        hits,false,misses,cnegs = calc_contingency_values(observation, bin_forecast)
    
        hits_random = float((hits + misses)*(hits + false))/float((hits + misses + false + cnegs))
        ets[i] = float((hits - hits_random))/float((hits + misses + false))

    return ets

def brier_skill_score( y_test, y_train, predictions ):
    """
    Calculate Brier Skill Score 
    """	
    brierScore = brier_score_loss( y_test, predictions )
    climo_value = (float( np.count_nonzero( y_train )) / float( len(y_train)))
	
    climo_fcst  = climo_value * np.ones(( np.shape(y_test) ))

    #climo_score is the BS that results from always forecasting climatology
    climo_score = brier_score_loss( y_test, climo_fcst )		

    BSS = ( climo_score - brierScore ) / climo_score

    return BSS, brierScore


#finds the brier skill score (BSS) when there are no explicit testing and training sets
#i.e., only testing/evaluation set (e.g., for real-time verification). 
#Big difference is that the evaluation set is used to compute climatology. 
#brierScore
def bss_no_train(y_vals, predictions):
    brierScore = brier_score_loss( y_vals, predictions )
    climo_value = (float( np.count_nonzero( y_vals )) / float( len(y_vals)))

    climo_fcst  = climo_value * np.ones(( np.shape(y_vals) ))

    #climo_score is the BS that results from always forecasting climatology
    climo_score = brier_score_loss( y_vals, climo_fcst )

    BSS = ( climo_score - brierScore ) / climo_score

    return BSS, brierScore

def binarize_prob_forecast( data, prob_threshold ):
    """
    Binarize forecast based on a given probability threshold 
    """
    bin_forecast = np.where(data >= prob_threshold, 1.0, 0.0)

    return bin_forecast


def calc_contingency_values( observation, bin_forecast ) :
    #a = hit; b = false ; c = miss ; d = cneg
    a = 0 ; b = 0 ; c = 0 ; d = 0
    #First find hits
    #find indices of yes observation
    yes_obs_inds = np.where(observation >=1) 
    no_obs_inds = np.where(observation == 0 ) 

    #hits are where we have yes obs and yes fcst
    a = sum(bin_forecast[yes_obs_inds])

    #misses are where we have yes obs and no fcst
    no_fcst = np.where(bin_forecast == 0, 1.0, 0.0) 
    c = sum(no_fcst[yes_obs_inds])
    
    #False alarms are where we have no obs and yes fcst 
    b = sum(bin_forecast[no_obs_inds])
    
    #Correct negatives are where we have no obs and no fcst
    d = sum(no_fcst[no_obs_inds])
    
    return a, b , c, d

def calc_POD( a, c ):
    """
    Calculate Probability of Detection 
    """
    if a == 0:
        return 0. 
    else: 
        return float(a) / (a+c)


def calc_POFD( b, d ):
    """
    Calculate Probability of False Detection 
    """
    if b == 0: 
        return 0. 
    else:
        return float(b) / (b+d)


def calc_SR(a, b):
    """
    Calculate Success Ration = hits/(hits + false)
    """

    if a == 0: 
        return 0.
    else: 
        return float(a)/(a + b) 

    return

#Ultimately returns the set of contingency table elements at each
#probability level in probs_auc
def obtainContingencySet(forecast_probs, observation):
    f_star = probs_auc
    hits = np.zeros(len(f_star))
    misses = np.zeros(len(f_star))
    false = np.zeros(len(f_star))
    cneg = np.zeros(len(f_star))

    for i in range(len(f_star)):
        prob_threshold = f_star[i]
        bin_forecast = binarize_prob_forecast(forecast_probs, prob_threshold)
        a, b, c, d = calc_contingency_values(observation, bin_forecast)
        hits[i] = a
        misses[i] = c
        false[i] = b
        cneg[i] = d

    return hits, misses, false, cneg


#computes POD, SR for variety of fcst probs. 
def perfDiag(forecast_probs, observation): 
    f_star = probs_auc
    pod = np.zeros((len(f_star)))
    sr = np.zeros((len(f_star)))
    
    for i in range(len(f_star)):
        prob_threshold = f_star[i]
        bin_forecast = binarize_prob_forecast(forecast_probs, prob_threshold)
        a, b, c, d = calc_contingency_values( observation, bin_forecast )
        pod[i]  = calc_POD( a, c )
        sr[i] = calc_SR(a,b) 

    return pod, sr


def rocCurve(forecast_probs, observation ): 
    #f_star = np.arange(0 , 1.01, 0.01)
    f_star = probs_auc
    pod = np.zeros(( len(f_star)))
    pofd = np.zeros(( len(f_star)))

    for i in range(len(f_star)):
        prob_threshold = f_star[i]
        bin_forecast = binarize_prob_forecast( forecast_probs, prob_threshold)
        a,b,c,d = calc_contingency_values( observation, bin_forecast )
        pod[i] = calc_POD( a, c )
        pofd[i] = calc_POFD( b, d )
    
    return pofd, pod 


#returns AUC value given a set of contingency table elements at various probability levels/thresholds
def getAUCfromContingencyElements(hits, misses, false, cnegs):

    #First need to sum the contingency elements over individual days
    #print "orig hits, misses, false, cnegs"
    #print hits
    #print misses
    #print false
    #print cnegs
    
    hits = np.sum(hits, axis=0)
    misses = np.sum(misses, axis=0)
    false = np.sum(false, axis=0)
    cnegs = np.sum(cnegs, axis=0)

    #print "summed contingency"
    #print hits
    #print misses
    #print false
    #print cnegs

    pod = np.zeros(len(hits))
    pofd = np.zeros(len(hits))


    for i in range(len(hits)):
        #print "hits, misses, false, correct"
        #print hhits[i], mmisses[i], ffalse[i], ccnegs[i]
        pod[i] = calc_POD(hits[i], misses[i])
        pofd[i] = calc_POFD(false[i], cnegs[i])

    auc = calcAUC(pod, pofd)

    return auc 

    


def calcAUC( pod, pofd ):

    return auc(pofd, pod, reorder=True)

    #return np.trapz( pod, pofd )


#NOTE: Must be careful to use this only with the rel probs
def BinSplit( forecast_probs, observations  ):
    BinSet = [ ]
    ObsSet = [ ]
    #threshold = np.arange(0, 1.1, 0.1)
    threshold = probs_rel
    print (threshold)
    for i in range(len(threshold)-1):  
        if threshold[i+1] == 1.0:
            #BinSet.append( forecast_probs[ np.where( (forecast_probs >= 0.9) & (forecast_probs <= 1.0) ) ] )
            #ObsSet.append( observations[ np.where( (forecast_probs >= 0.9) & (forecast_probs <= 1.0) ) ] )
            BinSet.append( forecast_probs[ np.where( (forecast_probs >= 0.95) & (forecast_probs <= 1.0) ) ] )
            ObsSet.append( observations[ np.where( (forecast_probs >= 0.95) & (forecast_probs <= 1.0) ) ] )
        else:
            BinSet.append( forecast_probs[ np.where( (forecast_probs >= threshold[i]) & (forecast_probs < threshold[i+1]) ) ] )
            ObsSet.append( observations[ np.where( (forecast_probs >= threshold[i]) & (forecast_probs < threshold[i+1]) ) ] )
        
    return BinSet, ObsSet

def ReliabilityCurve (forecast_probs, observations):

    BinSet, ObsSet = BinSplit(forecast_probs, observations )

    meanForecastProb = np.zeros(( len(probs_rel)-1 ))
    condEventFreq    = np.zeros(( len(probs_rel)-1 ))
        
    #Record number of forecasts per probability bin
    numForecasts = np.zeros((len(probs_rel)-1))

    for i in range(len(probs_rel)-1):
        meanForecastProb[i] = np.mean( BinSet[i] )
        numForecasts[i] = len(BinSet[i])
        if len(ObsSet[i]) == 0:
            condEventFreq[i] = np.nan
        else:
            condEventFreq[i] = float( np.count_nonzero( ObsSet[i])) / len( ObsSet[i] )

                
    return meanForecastProb, condEventFreq, numForecasts


