# This script is designed to do deep learning!
################################################

#Imports
import tensorflow as tf 
import numpy as np
import keras
from keras.layers import Input
from keras.optimizers import Adam, Adadelta
from keras.utils import multi_gpu_model
from keras.callbacks import EarlyStopping, ModelCheckpoint, ReduceLROnPlateau
from keras.models import load_model
import modelEvaluationFunctions_new as Eval
import os

#from separable_conv import * 
#from separable_conv_sseo_vars import * 

from separable_conv_severe_5x5 import *

##############
# Classes
##############

class AccuracyHistory(keras.callbacks.Callback):
    def on_train_begin(self, logs={}):
        self.acc = []

    def on_epoch_end(self, batch, logs={}):
        self.acc.append(logs.get('acc'))
        
        #EarlyStopping(monitor='acc', patience=1,)
        #ModelCheckpoint(filepath=model_outname, monitor='val_loss', save_best_only=True,)

class Metrics(keras.callbacks.Callback):
    def on_train_begin(self, logs={}):
        self._data = []

    def on_epoch_end(self, batch, logs={}):
        X_val, y_val = self.validation_data[0], self.validation_data[1]
        #keras_model = load_model(model_outname) 

        y_predict = np.asarray(self.model.predict(X_val))
        y_val = np.asarray(y_val) 


        #y_val = np.argmax(y_val, axis=1)
        #y_predict = np.argmax(y_predict, axis=1)

        bss = Eval.bss_no_train(y_val, y_predict)[0]
        print (bss) 

        self._data.append({
            'val_bss': bss,
        })
        

        return

    def get_data(self):
        return self._data

#metrics = Metrics()
#history = model.fit(X_train, y_train, epochs=100, validation_data=(X_val, y_val), callbacks=[metrics])
#metrics.get_data()


###############
# Functions
###############

#sess = tf.Session(config=tf.ConfigProto(log_device_placement=True))
#print (sess) 

def read_binary(infile, dx_pts, dy_pts, dsamples, dchannels):
    #@infile is the filename of unformatted binary file
    #Returns a numpy array of data
    #@dx_pts is an integer describing the number of longitude points
    #@dy_pts is an integer describing the number of latitude points

    f = open ( infile , 'rb' )
    arr = np.fromfile ( f , dtype = np.float32 , count = -1 )
    f.close()

    #Rebroadcast the data into 2-d array with proper format
    arr.shape = (dsamples, dy_pts, dx_pts, dchannels)
    return arr


def read_obs_binary(infile):
    #@infile is the filename of unformatted binary file
    #Returns a numpy array of data

    f = open ( infile , 'rb' )
    arr = np.fromfile ( f , dtype = np.float32 , count = -1 )
    f.close()

    return arr



#==========================
#User defined parameters
#==========================

#threshold = 2.54 # in mm
samp_rate = 1.0
n_mask_pts = 1434
patch_size = 5 #5x5 patches
n_channels = 12
samp_per_day = int(n_mask_pts*samp_rate)

#Not sure if we'll need these anymore
#n_days = 113
#
#samp_per_day = int(n_mask_pts*samp_rate) 
#total_samples = int(n_days*samp_per_day) 






#data_file = "preprocessed_ncar_data.npy"
#obs_file = "sample_obs.npy"

test_date_file = "test_dates.txt"
train_date_file = "train_dates.txt"
val_date_file = "val_dates.txt"

outdir_forecast = "./fcst"
outdir_observations = "./obs"

#data_file = "/glade/scratch/eloken/sseo/severe_DL/grib_dates/fcst_data/grib_fpatch.dat"
#obs_file = "/glade/scratch/eloken/sseo/severe_DL/grib_dates/obs_data/grib_obs.dat" 

#fdate_file = "ncar_sseo_grib_dates.txt"
#odate_file = "ncar_sseo_grib_obs_dates.txt"

model_outname = 'separable_conv_tutorial_5x5.h5'

#Hyperparameters
dropout_rate = 0.50
batch_size = 10000
n_epochs = 60
reg_penalty = 0.001 #regularization penalty
n_gpus = 2

#==========================

#Read in the data and obs

#NOTE: Let's employ a similar technique as we did previously 

test_dates = np.genfromtxt(test_date_file, dtype='str')
train_dates = np.genfromtxt(train_date_file, dtype='str')
val_dates = np.genfromtxt(val_date_file, dtype='str')

#Approach: Concatenate the training dates (fcst/obs) into training.dat and testing dates into testing.dat 
#Then read in the .dat files 
#Then train on the training set, test on the testing set -- for testing just output probability files

#NOTE: It's best to do this outside of casper so we don't waste the GPUs on this.
#print ("cancatenating files")
for d in train_dates:
    ffile = "%s/%s_fpatch.dat" %(outdir_forecast, d)
    ofile = "%s/%s_obs.dat" %(outdir_observations, d)
    os.system('cat %s >> training_fdata.dat' %ffile)
    os.system('cat %s >> training_odata.dat' %ofile)

for d in test_dates:
    ffile = "%s/%s_fpatch.dat" %(outdir_forecast, d)
    ofile = "%s/%s_obs.dat" %(outdir_observations, d)
    os.system('cat %s >> testing_fdata.dat' %ffile)
    os.system('cat %s >> testing_odata.dat' %ofile)

for d in val_dates:
    ffile = "%s/%s_fpatch.dat" %(outdir_forecast, d)
    ofile = "%s/%s_obs.dat" %(outdir_observations, d)
    os.system('cat %s >> val_fdata.dat' %ffile)
    os.system('cat %s >> val_odata.dat' %ofile)

print ("Reading in test/train data")
#Read in the data

training_x = read_binary('training_fdata.dat', patch_size, patch_size, samp_per_day*len(train_dates), n_channels)
testing_x = read_binary('testing_fdata.dat', patch_size, patch_size, samp_per_day*len(test_dates), n_channels)
val_x = read_binary('val_fdata.dat', patch_size, patch_size, samp_per_day*len(val_dates), n_channels)

training_y = read_obs_binary('training_odata.dat') 
testing_y = read_obs_binary('testing_odata.dat') 
val_y = read_obs_binary('val_odata.dat') 

print ("Climo of test set") 
print (sum(testing_y)/len(testing_y))

#NOTE: Important step: Need to tornamlize the data

train_mean = training_x.mean(axis=0)
train_sd = training_x.std(axis=0)

#Want to prevent divide by zero error
delta = 0.000001
train_sd += delta

np.save("train_mean.npy", train_mean)
np.save("train_std.npy", train_sd)

#Do the normalization
training_x = (training_x - train_mean)/train_sd
testing_x = (testing_x - train_mean)/train_sd
val_x = (val_x - train_mean)/train_sd



#Run the data/obs through keras model (which I'll put in another script)

input_image = Input((patch_size, patch_size, n_channels)) 

#Does not require multiple gpus.
conv_model = conv_net(input_image, dropout_rate, reg_penalty)

#Runs with multiple gpus
#parallel_model = multi_gpu_model(conv_net_model, gpus=n_gpus) 
#parallel_model.compile(optimizer=Adam(), loss='binary_crossentropy', metrics=['accuracy']) 

conv_model.compile(optimizer=Adam(), loss='binary_crossentropy', metrics=['accuracy'])

history = AccuracyHistory()

metrics = Metrics()
#history = model.fit(X_train, y_train, epochs=100, validation_data=(X_val, y_val), callbacks=[metrics])
#metrics.get_data()

#callbacks_list = [ EarlyStopping(monitor='acc', patience=3,), ModelCheckpoint(filepath=model_outname, monitor='val_loss', save_best_only=True,), ReduceLROnPlateau(monitor='val_loss', factor=0.1, patience=2), history]
callbacks_list = [ EarlyStopping(monitor='val_loss', patience=2,), ModelCheckpoint(filepath=model_outname, monitor='val_loss', save_best_only=True,), history, metrics]

#conv_net_model.fit(train_data, obs_train, batch_size=batch_size, epochs=n_epochs, verbose=1, validation_data=(val_data, obs_val), callbacks=[history])
#parallel_model.fit(training_x, training_y, batch_size=batch_size, epochs=n_epochs, verbose=1, validation_data=(val_x, val_y), callbacks=callbacks_list)
conv_model.fit(training_x, training_y, batch_size=batch_size, epochs=n_epochs, verbose=1, validation_data=(val_x, val_y), callbacks=callbacks_list)

print(metrics.get_data())

#score = parallel_model.evaluate(testing_x, testing_y, verbose=0)

score = conv_model.evaluate(testing_x, testing_y, verbose=0)

print ('Test loss:', score[0])
print ('Test accuracy:', score[1])
#print ('Test BSS:', score[2])
print ('Done!') 

#save model 
#parallel_model.save('simple_model.h5') 
#parallel_model.save('residual_model.h5')
#parallel_model.save('xception_model.h5')
#parallel_model.save('larger_depthwise_resnet.h5') 
#parallel_model.save('alt_depthwise_resnet.h5')
#parallel_model.save('new_depthwise_resnet_0.1in.h5') 
#parallel_model.save(model_outname)
#parallel_model.save(model_outname) 
conv_model.save(model_outname) 
#Remove the .dat files 
#os.system('rm training*.dat ; rm testing*.dat')
