#==========================================
# goal is to create a simple DL model to 
# see how well it performs
#===========================================


#Imports
import keras
from keras.models import Model
from keras import layers
from keras.layers import Dense, Dropout, Conv2D, BatchNormalization, Input, Flatten
from keras.layers import ZeroPadding2D, SeparableConv2D, DepthwiseConv2D, Cropping2D
from keras.layers.pooling import MaxPooling2D, AveragePooling2D
from keras import regularizers



#creates the convolutional neural network

#input_img is the input image with channels last
#drop_rate is the dropout rate (e.g., 0.3)
#penalty is the regularization penalty coeff
def conv_net(input_img, drop_rate, penalty):
   
    s1 = SeparableConv2D(24, kernel_size=2, activation='relu', padding='valid', kernel_regularizer=regularizers.l1(penalty), kernel_initializer='he_normal') (input_img)
    s1 = BatchNormalization() (s1)
    s1 = MaxPooling2D((2,2)) (s1) 
    s1 = Dropout(drop_rate) (s1) 

    s2 = SeparableConv2D(48, kernel_size=2, activation='relu', padding='valid', kernel_regularizer=regularizers.l1(penalty), kernel_initializer='he_normal') (s1)
    s2 = BatchNormalization() (s2)
    #s2 = MaxPooling2D((2,2)) (s2) 
    s2 = Dropout(drop_rate) (s2)

    #First residual layer

    r1 = SeparableConv2D(48, kernel_size=3, activation='relu', padding='valid', strides=1, kernel_regularizer=regularizers.l1(penalty), kernel_initializer='he_normal') (input_img)
    r1 = BatchNormalization() (r1)
    r1 = AveragePooling2D((2,2)) (r1) 
    r1 = Dropout(drop_rate) (r1)

    #Add the residual layer and main branch
    p1 = layers.add([s2, r1]) 

    #Now, flatten and use to make predictions
    f1 = Flatten() (p1) 
    f2 = Dense(36, activation='relu') (f1) 
    f2 = BatchNormalization() (f2) 
    f2 = Dropout(drop_rate) (f2) 

    #Second fully connected layer
    f3 = Dense(12, activation='relu') (f2) 
    f3 = BatchNormalization() (f3) 
    f3 = Dropout(drop_rate) (f3) 

    #Now get output layer
    outputs = Dense(1, activation='sigmoid') (f3) 

    model = Model(inputs=[input_img], outputs=[outputs])
    
    print (model.summary())

    return model




